package com.tofly.toflyServer.controller;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

import javax.servlet.http.HttpServletRequest;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.toflyServer.service.impl.AsyncService;
import com.tofly.toflyServer.utils.Props;
import com.tofly.toflyServer.utils.UrlUtil;
import com.tofly.toflyServer.utils.Utils;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import it.geosolutions.geoserver.rest.GeoServerRESTManager;
import it.geosolutions.geoserver.rest.GeoServerRESTPublisher;

@RestController
@RequestMapping("/dataSource")
@Api(tags = "数据存储管理")
public class DataSourceController {
	
	 @Autowired
	 private AsyncService asyncService;
	 
	 @Autowired
	 private Props props;
	 
	
	
	  /**
	    * 数据管理新增
	    * @param xml 数据管理新增
	    * @return ResultRespone
	    */
	   @ToFlyAppLog(title = "数据源新增")
	   @ApiOperation(value = "数据源新增 格式xml）")
	   @PostMapping("/addDataSource")
	   public ResultRespone addDataSource(@RequestBody String xml) {		   
		   
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        provider.setCredentials(scope, credentials);
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        HttpResponse response = null;	
	        
	        String serviceUrl = props.getGeoserverUrl();
	        
	    	try {		
				Document document = DocumentHelper.parseText(xml);
		        Element props = document.getRootElement();
		        List<Element> childElements = props.elements();//获取当前元素下的全部子元素		        
		        String workSpace = null;	        
		        for (Element child : childElements) {
		        	if(child.getName().equals("workSpace")) {
		        		workSpace = child.getText();
		        	}
		        }		        
		        if (workSpace == null  ) {
		            throw new Exception("参数传递不正确，请检查。");
		        }        			        
		        HttpPost httpPost = new HttpPost(serviceUrl+"/rest/workspaces/"+workSpace+"/datastores");	        
						
				StringEntity entity =new StringEntity(xml);
				entity.setContentType("application/xml");
				httpPost.setEntity(entity);
				response = closeableHttpClient.execute(httpPost);
	            // 关闭连接
	            closeableHttpClient.close();	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        } catch (Exception e) {
	            e.printStackTrace();
	        }
		    return ResultRespone.success(response.getStatusLine());
	    }
	   
	   
	   
//	   
//	    /**
//	     * 上传附件
//	     */
//	    @ToFlyAppLog(title = "上传附件至geoserver服务器")
//	    @ApiOperation(value = "上传附件至geoserver服务器")
//	    @PostMapping("/uploadFile")
//	    public ResultRespone uploadPublicHouseFile(HttpServletRequest servletRequest) {     
//	    	MultiValueMap<String, MultipartFile> multiFileMap = ((MultipartHttpServletRequest) servletRequest).getMultiFileMap();
//	    	String fileCreateId = new SimpleDateFormat("yyyyMMddHHmmssSSSS").format(System.currentTimeMillis());
//	    	String targetPath = "D:/dataSourceFile/"+fileCreateId+"/"; 
//	    	try {
//	    		for(Map.Entry<String, List<MultipartFile>> entry : multiFileMap.entrySet()) { 
//	                List<MultipartFile> v = entry.getValue();
//	                //后缀名
//	                String fileSuffix = v.get(0).getOriginalFilename().substring(v.get(0).getOriginalFilename().lastIndexOf("."));            
//	                //文件名
//	                String fileName = v.get(0).getOriginalFilename().substring(0, v.get(0).getOriginalFilename().lastIndexOf("."));
//  
//	                File targetFile = new File(targetPath);
//	        	    if (!targetFile.isDirectory()) {
//	        	    	targetFile.mkdirs();
//	        	    } 
//	        	    
//	        	    
//	        	    
//	        	    v.get(0).transferTo(new File(targetFile.getAbsolutePath() + "/" + fileName+fileSuffix));
//	    		}
//	    		UploadUtils uu = new UploadUtils("117.174.10.73",50922,"root","Huawei@5tgb");			 
//				uu.upload("/home/ogs/geoserver/data_dir/"+fileCreateId+"/",targetPath);		
//				uu.release();
//				uu.closeChannel();		
//	    	}catch (Exception e) {
//	    		e.printStackTrace();
//				return ResultRespone.failed(e.getMessage());
//			}
//	    	return ResultRespone.success("/home/ogs/geoserver/data_dir/"+fileCreateId);
//	    }
	    
	    
	    
	   @ToFlyAppLog(title = "数据存储列表")
	   @ApiOperation(value = "数据存储列表")
	   @GetMapping("/list")
	   public ResultRespone dataSourceList(HttpServletRequest servletRequest) {	
		   
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        provider.setCredentials(scope, credentials);
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();	        
	        
	        List<JSONObject> returnJson = new ArrayList<JSONObject>();  
	        
	        String url =props.getGeoserverUrl()+ "/rest/workspaces";
	       	        
	        try {
	        	HttpGet httpGet = new HttpGet(url);
	        	HttpResponse httpResponse = closeableHttpClient.execute(httpGet);
	        	HttpEntity entity = httpResponse.getEntity();
	            if( entity != null ){
	            	JSONObject resultJson = JSONObject.parseObject(EntityUtils.toString(entity));
	                JSONObject workspaces = resultJson.getJSONObject("workspaces");
	                JSONArray  workspace = workspaces.getJSONArray("workspace");  
	                
	                /**
	                 * 进行任务分配（分成3份，交付多线程执行）
	                 */
	                int perNum = (workspace.size() % 3 == 0) ? (workspace.size()/3) : (workspace.size()/3 +1);
	                List list = Utils.spliceArrays(workspace, perNum);

	                CompletableFuture<List<JSONObject>> resultNumOne = asyncService.getDataSourceJson(JSONArray.parseArray(list.get(0).toString()),url);
	                CompletableFuture<List<JSONObject>> resultNumTwo = asyncService.getDataSourceJson(JSONArray.parseArray(list.get(1).toString()),url);
	                CompletableFuture<List<JSONObject>> resultNumThree = asyncService.getDataSourceJson(JSONArray.parseArray(list.get(2).toString()),url);
		 	        CompletableFuture.allOf(resultNumOne,resultNumTwo,resultNumThree).join();
	                
		 	        returnJson.addAll(resultNumOne.get());
		 	        returnJson.addAll(resultNumTwo.get());
		 	        returnJson.addAll(resultNumThree.get());
	                
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        } catch (Exception e) {
	            e.printStackTrace();
	        }
	        
		    return ResultRespone.success(returnJson);
	    }
	    
	    
	   @ToFlyAppLog(title = "数据存储下图层列表")
	   @ApiOperation(value = "数据存储下图层列表  参数 （featureJsonUrl）")
	   @GetMapping("/dataSourceLayerList")
	   public ResultRespone dataSourceLayerList(HttpServletRequest servletRequest,@RequestParam String featureJsonUrl) {	
		   
		   // 创建HttpClientBuilder
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        // 设置BasicAuth
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        // Create the authentication scope
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        // Create credential pair，在此处填写用户名和密码
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        // Inject the credentials
	        provider.setCredentials(scope, credentials);
	        // Set the default credentials provider
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        // HttpClient
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        String result = "";
	        HttpGet httpGet = null;
	        HttpResponse httpResponse = null;
	        HttpEntity entity = null;
	        JSONObject resultJson =null;
	        JSONObject returnJson =null;
	        //返回数据源
	        List<JSONObject> resultList =new ArrayList<JSONObject>();
	        
	        httpGet = new HttpGet(featureJsonUrl);
	        try {
	        	httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                resultJson = JSONObject.parseObject(EntityUtils.toString(entity));
	                
	                JSONObject param = resultJson.getJSONObject("featureTypes");
	                if(null!=param) {
	                	JSONArray param1 = param.getJSONArray("featureType");
	  	                
	  	                for(int j=0;j<param1.size();j++){
	  	                    JSONObject job = param1.getJSONObject(j); 
	  	                    for (Map.Entry entry : job.entrySet()) {
	  		                	if(entry.getKey().toString().equals("href")) {
	  		                		String urlJson = UrlUtil.loadJson(entry.getValue().toString());
	  		                		returnJson = JSONObject.parseObject(urlJson);
	  		                	}
	  		                }	      
	  		                resultList.add(returnJson);
	  	                }
	                }
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
		    return ResultRespone.success(resultList);
	    }
	   
	   
	   
	   
	   /**
	    * 获取数据管理下图层发布情况
	    * @param dataSourceName
	    * @return
	    */
	   @ToFlyAppLog(title = "获取数据管理下图层发布情况")
	   @ApiOperation(value = "获取数据管理下图层发布情况")
	   @GetMapping("/publishStatus/{dataSourceName}")
	   public ResultRespone workSpaceList(@PathVariable("dataSourceName") String dataSourceName) {	

	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        UsernamePasswordCredentials credentials 
	        		= new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        provider.setCredentials(scope, credentials);
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        //返回数据源
	        List<JSONObject> resultList =new ArrayList<JSONObject>();	        
	        String workSpace = null;
	        String dataSource = null;	        
	    	String url = props.getGeoserverUrl();
	        try {
		        if(dataSourceName.indexOf(":")==-1) {
		        	throw new Exception("数据管理参数有误，请检查");
		        }else {
		        	workSpace = dataSourceName.substring(0,dataSourceName.indexOf(":"));
		        	dataSource  = dataSourceName.substring(dataSourceName.indexOf(":")+1,dataSourceName.length());
		        }
		        String noUseUrl =url+"/rest/workspaces/"+workSpace+"/datastores/"+dataSource+"/featuretypes.json?list=available";
		        String useUrl =url+"/rest/workspaces/"+workSpace+"/datastores/"+dataSource+"/featuretypes.json";
		        HttpGet httpGet = new HttpGet(useUrl);
	        	HttpResponse httpResponse = closeableHttpClient.execute(httpGet);
	        	HttpEntity entity  = httpResponse.getEntity();
	            if( entity != null ){
	            	String result = EntityUtils.toString(entity);
	            	JSONObject resultJson = JSONObject.parseObject(result);
	                JSONObject param = resultJson.getJSONObject("featureTypes");
	                if(null!=param) {
	                	JSONArray param1 = param.getJSONArray("featureType");
	  	                for(int j=0;j<param1.size();j++){	  
	  	                	JSONObject json = param1.getJSONObject(j);
	  	                	json.put("status", "true");
	  		                resultList.add(json);
	  	                }
	                }
	            }
	            HttpGet httpGet1 = new HttpGet(noUseUrl);
	            HttpResponse httpResponse1 = closeableHttpClient.execute(httpGet1);
	        	HttpEntity entity1  = httpResponse1.getEntity();
	            if( entity1 != null ){
	                String result1 = EntityUtils.toString(entity1);
	                JSONObject resultJson1 = JSONObject.parseObject(result1);
	                JSONObject param3 = resultJson1.getJSONObject("list");
	                if(null!=param3) {
	                	JSONArray param4 = param3.getJSONArray("string");
	  	                for(int k=0;k<param4.size();k++){	  
	  	                	JSONObject jsonNoUse = new JSONObject();
	  	                	jsonNoUse.put("status", "false");
	  	                	jsonNoUse.put("name", param4.getString(k));
	  		                resultList.add(jsonNoUse);
	  	                }
	                }
	            }
	            closeableHttpClient.close();
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        } catch (Exception e) {
	        	 e.printStackTrace();
	        }
		    return ResultRespone.success(resultList);
	    }
	   
  /**
	* 数据管理删除
	*
	* @param publicHouse 数据管理删除
	* @return ResultRespone
	*/
   @ToFlyAppLog(title = "数据管理删除")
   @ApiOperation(value = "数据管理删除 格式json，参数（workSpaceName：工作区名称，storeName：数据管理名称,isRecurse:是否递归删除 true|false）")
   @PostMapping("/deleteDataSource")
   public ResultRespone deleteDataSource(HttpServletRequest servletRequest,@RequestBody Map<String,String> dataSourceParams) {		   
	   	String url = props.getGeoserverUrl();
        String user = props.getGeoserverUsername();
        String password = props.getGeoserverPassword();	 
        // 连接geoServer
        GeoServerRESTManager geoServerRESTManager = null;
        try {	        	
		    if (dataSourceParams.get("workSpaceName") == null || dataSourceParams.get("storeName") == null || dataSourceParams.get("isRecurse") == null) {
		    throw new Exception("参数传递不正确，请检查。");
		    }	        	
			geoServerRESTManager = new GeoServerRESTManager(new URL(url), user, password);	            
			GeoServerRESTPublisher restPublisher = geoServerRESTManager.getPublisher();
			boolean delFlag = true;
			if(dataSourceParams.get("isRecurse").equals("true")) {
			delFlag = restPublisher.removeDatastore(dataSourceParams.get("workSpaceName"), dataSourceParams.get("storeName"),true);
			}else {
				delFlag = restPublisher.removeDatastore(dataSourceParams.get("workSpaceName"), dataSourceParams.get("storeName"),false);
			}		            
			if(delFlag) {
					return ResultRespone.success("删除成功");	          
			}else {
					return ResultRespone.failed("删除失败，请联系管理员");	          
	              }	            
	        } catch (Exception e) {
	            e.printStackTrace();
	            return ResultRespone.failed(e.getMessage());
	        }
   }
	   
}