package com.tofly.toflyServer.controller;

import java.io.IOException;
import java.net.URL;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.toflyServer.utils.Props;
import com.tofly.toflyServer.utils.UrlUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import it.geosolutions.geoserver.rest.GeoServerRESTManager;
import it.geosolutions.geoserver.rest.GeoServerRESTPublisher;

@RestController
@RequestMapping("/layerGroups")
@Api(tags = "图层组管理")
public class LayerGroupsController {
	
	 @Autowired
	 private Props props;
	
	   @ToFlyAppLog(title = "图层组列表")
	   @ApiOperation(value = "图层组列表")
	   @GetMapping("/list")
	   public ResultRespone workSpaceList(HttpServletRequest servletRequest) {	
		   	
		    // 创建HttpClientBuilder
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        // 设置BasicAuth
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        // Create the authentication scope
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        // Create credential pair，在此处填写用户名和密码
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        // Inject the credentials
	        provider.setCredentials(scope, credentials);
	        // Set the default credentials provider
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        // HttpClient
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        String result = "";
	        HttpGet httpGet = null;
	        HttpResponse httpResponse = null;
	        HttpEntity entity = null;
	        JSONObject resultJson =null;
	        
	        httpGet = new HttpGet(props.getGeoserverUrl()+"/rest/layergroups");
	        try {
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                result = EntityUtils.toString(entity);
	                resultJson = JSONObject.parseObject(result);
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	        
		    return ResultRespone.success(resultJson);
	    }
	   
	
	
	   @ToFlyAppLog(title = "图层组单个具体信息")
	   @ApiOperation(value = "图层组单个具体信息")
	   @GetMapping("/{name}")
	   public ResultRespone workSpaceByName(@PathVariable("name") String name) {	
		   	
		    // 创建HttpClientBuilder
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        // 设置BasicAuth
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        // Create the authentication scope
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        // Create credential pair，在此处填写用户名和密码
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        // Inject the credentials
	        provider.setCredentials(scope, credentials);
	        // Set the default credentials provider
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        // HttpClient
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        String result = "";
	        HttpGet httpGet = null;
	        HttpResponse httpResponse = null;
	        HttpEntity entity = null;
	        JSONObject resultJson =null;
	        
	        httpGet = new HttpGet(props.getGeoserverUrl()+"/rest/layergroups/"+name);
	        try {
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                result = EntityUtils.toString(entity);
	                resultJson = JSONObject.parseObject(result);
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	        
		    return ResultRespone.success(resultJson);
	    }
	   
	   
	   
		
	   @ToFlyAppLog(title = "图层组新增")
	   @ApiOperation(value = "图层组新增")
	   @PostMapping("/addLayerGroup")
	   public ResultRespone addLayerGroup(@RequestBody String xml) {	
		   	
		    // 创建HttpClientBuilder
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        // 设置BasicAuth
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        // Create the authentication scope
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        // Create credential pair，在此处填写用户名和密码
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        // Inject the credentials
	        provider.setCredentials(scope, credentials);
	        // Set the default credentials provider
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        // HttpClient
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        HttpPost httpPost = new HttpPost(props.getGeoserverUrl()+"/rest/layergroups");	        
	        //添加body
			HttpResponse response = null;
			
			try {
				StringEntity entity =new StringEntity(xml);
				entity.setContentType("application/xml");
				httpPost.setEntity(entity);
	        
				response = closeableHttpClient.execute(httpPost);
				
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	        
		    return ResultRespone.success(response.getStatusLine());
	    }
	   
	   
	   
	   /**
	    * 图层组删除
	    *
	    * @param publicHouse 图层删除
	    * @return ResultRespone
	    */
	   @ToFlyAppLog(title = "图层组删除")
	   @ApiOperation(value = "图层组删除 格式json，参数（workSpaceName：工作区名称，layerGroupName：图层组名称）")
	   @PostMapping("/{deleteLayerGroup}")
	   public ResultRespone deleteLayerGroup(HttpServletRequest servletRequest,@RequestBody Map<String,String> dataSourceParams) {		   
		   	String url = props.getGeoserverUrl();
	        String user = props.getGeoserverUsername();
	        String password = props.getGeoserverPassword();	 
	        // 连接geoServer
	        GeoServerRESTManager geoServerRESTManager = null;
	        try {
	        	
	            if (dataSourceParams.get("workSpaceName") == null || dataSourceParams.get("layerGroupName") == null ) {
			            throw new Exception("参数传递不正确，请检查。");
			    }
	        		        	
	            geoServerRESTManager = new GeoServerRESTManager(new URL(url), user, password);	            
	            GeoServerRESTPublisher restPublisher = geoServerRESTManager.getPublisher();
	            
	            boolean delFlag = restPublisher.removeLayerGroup(dataSourceParams.get("workSpaceName"), dataSourceParams.get("layerGroupName"));
	      
	            if(delFlag) {
	            	return ResultRespone.success("删除成功");	          
	            }else {
	            	return ResultRespone.failed("删除失败，请联系管理员");	          
	            }
	            
	        } catch (Exception e) {
	            e.printStackTrace();
	            return ResultRespone.failed(e.getMessage());
	        }
	   }
	
}
