package com.tofly.toflyServer.controller;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

import javax.servlet.http.HttpServletRequest;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mysql.cj.x.protobuf.MysqlxDatatypes.Object;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.toflyServer.service.impl.AsyncService;
import com.tofly.toflyServer.utils.Props;
import com.tofly.toflyServer.utils.UrlUtil;
import com.tofly.toflyServer.utils.Utils;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import it.geosolutions.geoserver.rest.GeoServerRESTManager;
import it.geosolutions.geoserver.rest.GeoServerRESTPublisher;

@RestController
@RequestMapping("/layers")
@Api(tags = "图层管理")
public class LayersController {
	
	   @Autowired
	   private AsyncService asyncService;
	   
	   @Autowired
	   private Props props;
	
	   /**
	    * 多线程获取图层列表
	    * @param servletRequest
	    * @return
	    */
	   @ToFlyAppLog(title = "图层列表状态")
	   @ApiOperation(value = "图层列表状态")
	   @GetMapping("/list")
	   public ResultRespone layerList(HttpServletRequest servletRequest) {	
		   
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        provider.setCredentials(scope, credentials);
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();	        
	        
	        List<JSONObject> returnJson = new ArrayList<JSONObject>();        
	        String url = props.getGeoserverUrl()+"/rest/layers";
	       	        
	        try {
	        	HttpGet httpGet = new HttpGet(url);
	        	HttpResponse httpResponse = closeableHttpClient.execute(httpGet);
	        	HttpEntity entity = httpResponse.getEntity();
	            if( entity != null ){
	            	JSONObject resultJson = JSONObject.parseObject(EntityUtils.toString(entity));
	                JSONObject layers = resultJson.getJSONObject("layers");
	                JSONArray  layer = layers.getJSONArray("layer"); 
	                
	                
	                /**
	                 * 进行任务分配（分成3份，交付多线程执行）
	                 */
	                int perNum = (layer.size() % 3 == 0) ? (layer.size()/3) : (layer.size()/3 +1);
	                List list = Utils.spliceArrays(layer, perNum);
	                	                
	                CompletableFuture<List<JSONObject>> resultNumOne = asyncService.getLayerJson(JSONArray.parseArray(list.get(0).toString()),url);
	                CompletableFuture<List<JSONObject>> resultNumTwo = asyncService.getLayerJson(JSONArray.parseArray(list.get(1).toString()),url);
	                CompletableFuture<List<JSONObject>> resultNumThree = asyncService.getLayerJson(JSONArray.parseArray(list.get(2).toString()),url);
		 	        CompletableFuture.allOf(resultNumOne,resultNumTwo,resultNumThree).join();
		 	        
		 	        returnJson.addAll(resultNumOne.get());
		 	        returnJson.addAll(resultNumTwo.get());
		 	        returnJson.addAll(resultNumThree.get());
		 	        
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        } catch (Exception e) {
	            e.printStackTrace();
	        }
	        
		    return ResultRespone.success(returnJson);
	    }
	   
	   @ToFlyAppLog(title = "图层具体信息")
	   @ApiOperation(value = "图层具体信息")
	   @GetMapping("/{name}")
	   public ResultRespone layerByName(@PathVariable("name") String name) {	
		   	
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        provider.setCredentials(scope, credentials);
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        String result = "";
	        HttpGet httpGet = null;
	        HttpResponse httpResponse = null;
	        HttpEntity entity = null;
	        JSONObject resultJson =null;
	        JSONObject returnJson =null;
	        
	        httpGet = new HttpGet(props.getGeoserverUrl()+"/rest/layers/"+name);
	        try {
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                result = EntityUtils.toString(entity);
	                resultJson = JSONObject.parseObject(result);
	                
	                JSONObject param = resultJson.getJSONObject("layer");
	                JSONObject  param1 = param.getJSONObject("resource");
	                for (Map.Entry entry : param1.entrySet()) {
	                	
	                	if(entry.getKey().toString().equals("href")) {
	                		String urlJson = UrlUtil.loadJson(entry.getValue().toString());
	                		returnJson = JSONObject.parseObject(urlJson);
	                	}
	                }	                	                
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
		    return ResultRespone.success(returnJson);
	    }
	   /**
	    * 图层删除
	    *
	    * @param publicHouse 图层删除
	    * @return ResultRespone
	    */
	   @ToFlyAppLog(title = "图层删除")
	   @ApiOperation(value = "图层删除 格式json，参数（workSpaceName：工作区名称，layerName：图层名称）")
	   @PostMapping("/deleteLayer")
	   public ResultRespone deleteLayer(HttpServletRequest servletRequest,@RequestBody Map<String,String> dataSourceParams) {		   
		   	String url = props.getGeoserverUrl();
	        String user = props.getGeoserverUsername();
	        String password = props.getGeoserverPassword();	 
	        // 连接geoServer
	        GeoServerRESTManager geoServerRESTManager = null;
	        try {
	            if (dataSourceParams.get("workSpaceName") == null || dataSourceParams.get("layerName") == null ) {
			            throw new Exception("参数传递不正确，请检查。");
			    }	        	
	            geoServerRESTManager = new GeoServerRESTManager(new URL(url), user, password);	            
	            GeoServerRESTPublisher restPublisher = geoServerRESTManager.getPublisher();
	            boolean delFlag = restPublisher.removeLayer(dataSourceParams.get("workSpaceName"), dataSourceParams.get("layerName"));
	            if(delFlag) {
	            	 return ResultRespone.success("删除成功");	          
	            }else {
	            	return ResultRespone.failed("删除失败，请联系管理员");	          
	            }
	        } catch (Exception e) {
	            e.printStackTrace();
	            return ResultRespone.failed(e.getMessage());
	        }
	   }
}