package com.tofly.toflyServer.controller;

import java.io.File;
import java.io.FileOutputStream;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.toflyServer.service.impl.AsyncService;
import com.tofly.toflyServer.utils.Props;
import com.tofly.toflyServer.utils.ShapeFileUtil;
import com.tofly.toflyServer.utils.UploadUtils;
import com.tofly.toflyServer.utils.ZipUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import it.geosolutions.geoserver.rest.GeoServerRESTManager;
import it.geosolutions.geoserver.rest.GeoServerRESTPublisher;
import it.geosolutions.geoserver.rest.GeoServerRESTPublisher.UploadMethod;
import it.geosolutions.geoserver.rest.decoder.RESTDataStore;
import it.geosolutions.geoserver.rest.encoder.GSLayerEncoder;
import it.geosolutions.geoserver.rest.encoder.GSResourceEncoder.ProjectionPolicy;
import it.geosolutions.geoserver.rest.encoder.feature.GSFeatureTypeEncoder;

@RestController
@RequestMapping("/servicePublish")
@Api(tags = "图层发布管理")
public class ServicePublishController {
	
	
	   @Autowired
	   private ShapeFileUtil shapeFileUtil;
	   
	   @Autowired
	   private Props props;
	
	
	   /**
	    * 发布shape数据聚
	    * @return ResultRespone
	    */
	   @ToFlyAppLog(title = "shape图层发布")
	   @ApiOperation(value = "shape图层发布 body参数 workSpace：工作空间、storeName:数据管理、"
	   		+ "datasetName：要发布的数据集名称、srs：坐标系、nativeSrs：数据源坐标系、title：图层标题、name：命名、nativeName：图层名")
	   @PostMapping("/publishShape")
	   public ResultRespone publishShape(HttpServletRequest servletRequest,
			   @RequestBody Map<String,String> shapeParams) {		     	    	
		   GeoServerRESTManager manager =null;
		   try {
			    if (shapeParams.get("workSpace") == null || shapeParams.get("storeName") == null || shapeParams.get("datasetName") == null 
					   || shapeParams.get("srs") == null  || shapeParams.get("nativeSrs") == null || shapeParams.get("title") == null
					   || shapeParams.get("name") == null || shapeParams.get("nativeName") == null) {
		            throw new Exception("参数为空，请检查。");
		        }
		  
			   	String url = props.getGeoserverUrl();
		        String user = props.getGeoserverUsername();
		        String password = props.getGeoserverPassword();	 
		        
			    URL u = new URL(url);	
			    manager = new GeoServerRESTManager(u, user, password);	
			    RESTDataStore ds= manager.getReader().getDatastore(shapeParams.get("workSpace"), shapeParams.get("storeName"));
			    
			    if(null!=ds) {
			    	  String absoluteUrl = "/home/ogs/geoserver/data_dir/";			    	
			    	  String dataUrl = ds.getConnectionParameters().get("url");			    	  
			    	  String sourceUrl = null;
			    	  
			    	  if(dataUrl.indexOf(":/")!=-1) {
			    		   sourceUrl=  dataUrl.substring(dataUrl.indexOf(":")+1, dataUrl.lastIndexOf("/")+1);
			    	  }else {
			    		   sourceUrl=  dataUrl.substring(dataUrl.indexOf(":")+1, dataUrl.lastIndexOf("/")+1);
			    		   sourceUrl=  absoluteUrl + sourceUrl;
			    	  }

			    	  String fileCreateId = new SimpleDateFormat("yyyyMMddHHmmssSSSS").format(System.currentTimeMillis());
			    	  String targetPath = "d:/downPath/"+fileCreateId+"/";		    	  
			    	  String zipPath = "d:/downPath/";		    	  
			    	  UploadUtils uu = new UploadUtils("117.174.10.73",50922,"root","Huawei@5tgb");			 
			    	  uu.recursiveFolderDownload(sourceUrl,targetPath);		
			    	  uu.release();
			    	  uu.closeChannel();	
			    	  
			    	  File upladFile = new File(zipPath+fileCreateId+".zip");
			    	  
			    	  /** 压缩方法 */
				  	  FileOutputStream fos1 = new FileOutputStream(upladFile);
				  	  ZipUtil.toZip(targetPath,fos1,false);
				  	  
				  	  shapeFileUtil.publishShp(shapeParams.get("workSpace"),  shapeParams.get("storeName"),null,
				  			 shapeParams.get("datasetName"),UploadMethod.FILE,upladFile.toURI(), shapeParams.get("srs"),
		            		 shapeParams.get("nativeSrs"), ProjectionPolicy.FORCE_DECLARED,null,shapeParams.get("name"),shapeParams.get("title"),
		            		 shapeParams.get("nativeName"));
				  	  
				      return ResultRespone.success("发布成功");
			    }else {
			    	throw new Exception("数据集不存在，请重新输入。");
			    }
			    
	    	}catch (Exception e) {
				return ResultRespone.failed(e.getMessage());
			}
	   }
	   
	   /**
	    * 发布postgis图层
	    * @return ResultRespone
	    */
	   @ToFlyAppLog(title = "发布postgis图层")
	   @ApiOperation(value = "发布postgis图层 body参数 workSpace：工作空间、storeName:数据管理、"
	   		+ "datasetName：要发布的数据集名称、srs：坐标系、nativeSrs：数据源坐标系、title：图层标题、name：命名")
	   @PostMapping("/publishPostgis")
	   public ResultRespone publishPostgis(HttpServletRequest servletRequest,
			   @RequestBody Map<String,String> shapeParams) {		     	    	
		   GeoServerRESTManager manager =null;
		   try {
			    if (shapeParams.get("workSpace") == null || shapeParams.get("storeName") == null || shapeParams.get("datasetName") == null 
					   || shapeParams.get("srs") == null  || shapeParams.get("nativeSrs") == null || shapeParams.get("title") == null
					   || shapeParams.get("name") == null ) {
		            throw new Exception("参数为空，请检查。");
		        }
		  
			   	String url = props.getGeoserverUrl();
		        String user = props.getGeoserverUsername();
		        String password = props.getGeoserverPassword();	 
		        
			    URL u = new URL(url);	
			    manager = new GeoServerRESTManager(u, user, password);	
			    RESTDataStore ds= manager.getReader().getDatastore(shapeParams.get("workSpace"), shapeParams.get("storeName"));
			    GeoServerRESTPublisher restPublisher = manager.getPublisher();
			    
			    
			    if(null!=ds) {
			    	
			    	GSFeatureTypeEncoder fteEncoder =new  GSFeatureTypeEncoder();
			    	
			    	fteEncoder.setName(shapeParams.get("name"));
			    	fteEncoder.setTitle(shapeParams.get("title"));
			    	fteEncoder.setNativeName(shapeParams.get("datasetName"));
			    	fteEncoder.setSRS(shapeParams.get("srs"));
			    	fteEncoder.setNativeCRS(shapeParams.get("nativeSrs"));
			    	
			    	final GSLayerEncoder layerEncoder = shapeFileUtil.configureDefaultStyle(null);
			    	
			    	restPublisher.publishDBLayer(shapeParams.get("workSpace"), shapeParams.get("storeName") ,
			    			fteEncoder, layerEncoder);
				  	  
				    return ResultRespone.success("发布成功");
			    }else {
			    	throw new Exception("数据集不存在，请重新输入。");
			    }
			    
	    	}catch (Exception e) {
				return ResultRespone.failed(e.getMessage());
			}
	   }
}