package com.tofly.toflyServer.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.toflyServer.utils.FileUtil;
import com.tofly.toflyServer.utils.Props;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping("/system")
@Api(tags = "服务器状态管理")
public class SystemController {
	
	   @Autowired
	   private Props props;
	
	   @ToFlyAppLog(title = "服务器状态")
	   @ApiOperation(value = "服务器状态")
	   @GetMapping("/systemStatus")
	   public ResultRespone systemStatus(HttpServletRequest servletRequest) {	
		   	
		   // 创建HttpClientBuilder
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        // 设置BasicAuth
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        // Create the authentication scope
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        // Create credential pair，在此处填写用户名和密码
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        // Inject the credentials
	        provider.setCredentials(scope, credentials);
	        // Set the default credentials provider
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        // HttpClient
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        String result = "";
	        HttpGet httpGet = null;
	        HttpResponse httpResponse = null;
	        HttpEntity entity = null;
	        JSONObject resultJson =null;
	        
	        httpGet = new HttpGet(props.getGeoserverUrl()+"/rest/about/system-status");
	        try {
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                result = EntityUtils.toString(entity);
	                resultJson = JSONObject.parseObject(result);
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	        
		    return ResultRespone.success(resultJson);
	    }
	   
	   
	   
	   @ToFlyAppLog(title = "服务器模块状态")
	   @ApiOperation(value = "服务器模块状态")
	   @GetMapping("/moudleStatus")
	   public ResultRespone moudleStatus(HttpServletRequest servletRequest) {	
		   	
		   // 创建HttpClientBuilder
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        // 设置BasicAuth
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        // Create the authentication scope
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        // Create credential pair，在此处填写用户名和密码
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        // Inject the credentials
	        provider.setCredentials(scope, credentials);
	        // Set the default credentials provider
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        // HttpClient
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        String result = "";
	        HttpGet httpGet = null;
	        HttpResponse httpResponse = null;
	        HttpEntity entity = null;
	        JSONObject resultJson =null;
	        Document doc = null;
	        List<Map<String,String>>  valueList =new ArrayList<Map<String,String>>();
	        httpGet = new HttpGet(props.getGeoserverUrl()+"/rest/about/status");
	        try {
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                result = EntityUtils.toString(entity);
	                doc = Jsoup.parse(result);
	                
	                
	                Elements ElementsLi = doc.select("li"); 
	    			for (Element elementUl : ElementsLi) {
	    				HashMap<String,String> valMap =new HashMap<String,String>();
	    				Elements nameEl = elementUl.getElementsByTag("h3");
    					for (Element h3 : nameEl) {
    						String moduleName = h3.text();
    						valMap.put("ModuleName", moduleName.substring(moduleName.indexOf(":")+1, moduleName.length()).trim());
    					}
	    				
	    				
	    				Elements liEl = elementUl.getElementsByTag("ul");
	    				for (Element element : liEl) {
	    					
	    					Elements valEl = element.getElementsByTag("li");
	    					for (Element li : valEl) {
	    						String categroyName = li.text();
	    						if(categroyName.indexOf("Module")!=-1) {
	    							valMap.put("Module", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Version")!=-1) {
	    							valMap.put("Version", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Enabled")!=-1) {
	    							valMap.put("Enabled", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Available")!=-1) {
	    							valMap.put("Available", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Message")!=-1) {
	    							valMap.put("Message", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Component")!=-1) {
	    							valMap.put("Component", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    					}
	    					valueList.add(valMap);
	    				}
	    			}	             
	                //resultJson = JSONObject.parseObject(result);
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        } catch (Exception e) {
	            e.printStackTrace();
	        }
		    return ResultRespone.success(valueList);
	    }
	   
	   
	   
	   
	   
	   @ToFlyAppLog(title = "服务器配置状态")
	   @ApiOperation(value = "服务器配置状态")
	   @GetMapping("/manifest")
	   public ResultRespone manifest(HttpServletRequest servletRequest) {	
		   	
		   // 创建HttpClientBuilder
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        // 设置BasicAuth
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        // Create the authentication scope
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        // Create credential pair，在此处填写用户名和密码
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        // Inject the credentials
	        provider.setCredentials(scope, credentials);
	        // Set the default credentials provider
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        // HttpClient
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        String result = "";
	        HttpGet httpGet = null;
	        HttpResponse httpResponse = null;
	        HttpEntity entity = null;
	        JSONObject resultJson =null;
	        Document doc = null;
	        List<Map<String,String>>  valueList =new ArrayList<Map<String,String>>();
	        httpGet = new HttpGet(props.getGeoserverUrl()+"/rest/about/manifest");
	        try {
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                result = EntityUtils.toString(entity);
	                doc = Jsoup.parse(result);
	                
	                
	                Elements ElementsLi = doc.select("li"); 
	    			for (Element elementUl : ElementsLi) {
	    				HashMap<String,String> valMap =new HashMap<String,String>();
	    				Elements nameEl = elementUl.getElementsByTag("h3");
    					for (Element h3 : nameEl) {
    						String moduleName = h3.text();
    						valMap.put("ModuleName", moduleName.substring(moduleName.indexOf(":")+1, moduleName.length()).trim());
    					}
	    				
	    				
	    				Elements liEl = elementUl.getElementsByTag("ul");
	    				for (Element element : liEl) {
	    					
	    					Elements valEl = element.getElementsByTag("li");
	    					for (Element li : valEl) {
	    						String categroyName = li.text();
	    						if(categroyName.indexOf("Module")!=-1) {
	    							valMap.put("Module", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Version")!=-1) {
	    							valMap.put("Version", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Enabled")!=-1) {
	    							valMap.put("Enabled", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Available")!=-1) {
	    							valMap.put("Available", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Message")!=-1) {
	    							valMap.put("Message", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    						if(categroyName.indexOf("Component")!=-1) {
	    							valMap.put("Component", categroyName.substring(categroyName.indexOf(":")+1, categroyName.length()).trim());
	    						}
	    					}
	    					valueList.add(valMap);
	    				}
	    			}	             
	                //resultJson = JSONObject.parseObject(result);
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        } catch (Exception e) {
	            e.printStackTrace();
	        }
		    return ResultRespone.success(valueList);
	    }
	   

	   @ToFlyAppLog(title = "获取服务器文件路径信息")
	   @ApiOperation(value = "获取服务器文件路径信息  filePath（地址绝对路径 比如 /home/ogs/geoserver/data_dir/data/sf/） ")
	   @PostMapping("/getFiles")
	   public ResultRespone getFiles(@RequestBody Map<String,String> filePath) {	
		    List<HashMap<String,String>> result = null;
		   	try {
		   		result =FileUtil.getSubfolderName("117.174.10.73", "root", "Huawei@5tgb", filePath.get("filePath"));            
		    }catch (Exception e) {
	            e.printStackTrace();
	        }
		    return ResultRespone.success(result);
	   }
	   
	   @ToFlyAppLog(title = "获取geoserver服务器地址")
	   @ApiOperation(value = "获取geoserver服务器地址")
	   @PostMapping("/getSystemUrl")
	   public ResultRespone getSystemUrl(HttpServletRequest servletRequest) {			   
		    return ResultRespone.success(props.getGeoserverUrl());
	   }
	   
	   
}