package com.tofly.toflyServer.controller;

import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.toflyServer.utils.Props;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import it.geosolutions.geoserver.rest.GeoServerRESTManager;
import it.geosolutions.geoserver.rest.GeoServerRESTPublisher;
import it.geosolutions.geoserver.rest.GeoServerRESTReader;
import it.geosolutions.geoserver.rest.decoder.RESTCoverageStoreList;
import it.geosolutions.geoserver.rest.decoder.RESTDataStoreList;
import it.geosolutions.geoserver.rest.decoder.RESTLayerList;

@RestController
@RequestMapping("/workSpace")
@Api(tags = "工作区管理")
public class WorkSpaceController {

	   @Autowired
	   private Props props;
	
	   /**
	    * 工作区新增
	    *
	    * @param publicHouse 工作区新增
	    * @return ResultRespone
	    */
	   @ToFlyAppLog(title = "工作区新增")
	   @ApiOperation(value = "工作区新增 格式json ，参数（name：工作区名称 ，url：命名空间）")
	   @PostMapping
	   public ResultRespone addWorkSpace(HttpServletRequest servletRequest,@RequestBody Map<String,String> workSpaceParams) {		   
		    String url = props.getGeoserverUrl();
	        String user = props.getGeoserverUsername();
	        String password = props.getGeoserverPassword();	 
	        
	       
	 
	        // 连接geoServer
	        GeoServerRESTManager geoServerRESTManager = null;
	        try {
	            geoServerRESTManager = new GeoServerRESTManager(new URL(url), user, password);
	            
	            GeoServerRESTReader restReader = geoServerRESTManager.getReader();
	            GeoServerRESTPublisher restPublisher = geoServerRESTManager.getPublisher();
	            
	            String workSpace = workSpaceParams.get("name");
	            // 存在相应的工作区
	            if (!restReader.existsWorkspace(workSpace)) {
	                restPublisher.createWorkspace(workSpace);
	            }else {
	            	throw new Exception("工作空间已存在，请重新输入");
	            }
	        } catch (Exception e) {
	            e.printStackTrace();
	            return ResultRespone.failed(e.getMessage());
	        }
	        return ResultRespone.success();
	   }
	   
	   
	   
	   
	   @ToFlyAppLog(title = "工作区列表")
	   @ApiOperation(value = "工作区列表")
	   @GetMapping("/list")
	   public ResultRespone workSpaceList(HttpServletRequest servletRequest) {	
		   	
		    // 创建HttpClientBuilder
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        // 设置BasicAuth
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        // Create the authentication scope
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        // Create credential pair，在此处填写用户名和密码
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        // Inject the credentials
	        provider.setCredentials(scope, credentials);
	        // Set the default credentials provider
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        // HttpClient
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        String result = "";
	        HttpGet httpGet = null;
	        HttpResponse httpResponse = null;
	        HttpEntity entity = null;
	        JSONObject resultJson =null;
	        
	        httpGet = new HttpGet(props.getGeoserverUrl()+"/rest/workspaces");
	        	        
	        try {
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                result = EntityUtils.toString(entity);
	                resultJson = JSONObject.parseObject(result);
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	        
		    return ResultRespone.success(resultJson);
	    }
	   
	   
	   @ToFlyAppLog(title = "工作区具体信息")
	   @ApiOperation(value = "工作区具体信息")
	   @GetMapping("/{name}")
	   public ResultRespone workSpaceList(@PathVariable("name") String name) {	
		   	
		    // 创建HttpClientBuilder
	        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
	        // 设置BasicAuth
	        CredentialsProvider provider = new BasicCredentialsProvider();
	        // Create the authentication scope
	        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
	        // Create credential pair，在此处填写用户名和密码
	        UsernamePasswordCredentials credentials = 
	        		new UsernamePasswordCredentials(props.getGeoserverUsername(), props.getGeoserverPassword());
	        // Inject the credentials
	        provider.setCredentials(scope, credentials);
	        // Set the default credentials provider
	        httpClientBuilder.setDefaultCredentialsProvider(provider);
	        // HttpClient
	        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
	        
	        String result = "";
	        HttpGet httpGet = null;
	        HttpResponse httpResponse = null;
	        HttpEntity entity = null;
	        JSONObject resultJson =null;
	        
	        httpGet = new HttpGet(props.getGeoserverUrl()+"/rest/workspaces/"+name);
	        try {
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                result = EntityUtils.toString(entity);
	                resultJson = JSONObject.parseObject(result);
	            }
	            // 关闭连接
	            closeableHttpClient.close();
	            
	        } catch (ClientProtocolException e) {
	            e.printStackTrace();
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	        
		    return ResultRespone.success(resultJson);
	    }
	   
	   /**
	    * 工作区删除
	    *
	    * @param publicHouse 工作区删除
	    * @return ResultRespone
	    */
	   @ToFlyAppLog(title = "工作区删除")
	   @ApiOperation(value = "工作区新增 格式json ，参数（name：工作区名称,isRecurse:是否递归删除 true|false）")
	   @PostMapping("/deleteWorkSpace")
	   public ResultRespone deleteWorkSpace(HttpServletRequest servletRequest,@RequestBody Map<String,String> workSpaceParams) {		   
		    String url = props.getGeoserverUrl();
	        String user = props.getGeoserverUsername();
	        String password = props.getGeoserverPassword();	 
	        // 连接geoServer
	        GeoServerRESTManager geoServerRESTManager = null;
	        try {
	            if (workSpaceParams.get("name") == null || workSpaceParams.get("isRecurse") == null ) {
			            throw new Exception("参数传递不正确，请检查。");
			    }        	
	            geoServerRESTManager = new GeoServerRESTManager(new URL(url), user, password);	            
	            GeoServerRESTPublisher restPublisher = geoServerRESTManager.getPublisher();
	            boolean delFlag = true;
	            if(workSpaceParams.get("isRecurse").equals("true")) {
	            	delFlag = restPublisher.removeWorkspace(workSpaceParams.get("name"), true);
	            }else {
	            	delFlag = restPublisher.removeWorkspace(workSpaceParams.get("name"), false);
	            }	
	            if(delFlag) {
	            	 return ResultRespone.success("删除成功");	          
	            }else {
	            	return ResultRespone.failed("删除失败，请联系管理员");	          
	            }
	        } catch (Exception e) {
	            e.printStackTrace();
	            return ResultRespone.failed(e.getMessage());
	        }
	   }
	   
	   /**
	    * 获取工作区下所有信息
	    *
	    * @param publicHouse 获取工作区下所有信息
	    * @return ResultRespone
	    */
	   @ToFlyAppLog(title = "获取工作区下所有信息")
	   @ApiOperation(value = "获取工作区下所有信息 格式json ，参数（workSpaceName：工作区名称,以逗号隔开。）")
	   @PostMapping("/queryWorkSpace")
	   public ResultRespone queryWorkSpace(HttpServletRequest servletRequest,@RequestBody Map<String,String> workSpaceParams) {		   
		    String url = props.getGeoserverUrl();
	        String user = props.getGeoserverUsername();
	        String password = props.getGeoserverPassword();	 
	        // 连接geoServer
	        GeoServerRESTManager geoServerRESTManager = null;
	        HashMap<String,Object>   returnMaps =new HashMap<String,Object>();
	        try {
	            if (workSpaceParams.get("workSpaceName") == null  ) {
			            throw new Exception("参数传递不正确，请检查。");
			    }        	
	            String[] workSpaces = workSpaceParams.get("workSpaceName").split(",");
	            
	            for(String ws:workSpaces) {
	            	HashMap<String,Object>   returnMap =new HashMap<String,Object>();
	            	geoServerRESTManager = new GeoServerRESTManager(new URL(url), user, password);	            
		            GeoServerRESTReader restReader = geoServerRESTManager.getReader();
		           
		            returnMap.put("dataStores", restReader.getDatastores(ws));
		            returnMap.put("layerGroups",  restReader.getLayerGroups(ws));
		            returnMap.put("styles", restReader.getStyles(ws));
		            returnMap.put("featureTypes", restReader.getFeatureTypes(ws));
		            
		            RESTLayerList list = restReader.getLayers();	            
		            HashMap<String,Object> layersMap =new HashMap<String,Object>();
		            List<String> layerName =new ArrayList<String>();
		                       
		            for(int i=0;i<list.size();i++) {
		            	if(list.get(i).getName().indexOf(ws)!=-1) {
		            		layerName.add(list.get(i).getName());
		            	}    	
		            }
		            if(layerName.size()==0) {
		            	 returnMap.put("empty","true");  
		            	 returnMap.put("layers",layerName);  
		            }else {
		            	 returnMap.put("empty","false");  
		            	 returnMap.put("layers",layerName);  
		            }
		            returnMaps.put(ws, returnMap);
	            }
	            return ResultRespone.success(returnMaps);
	        } catch (Exception e) {
	            e.printStackTrace();
	            return ResultRespone.failed(e.getMessage());
	        }
	   }
}