package com.tofly.toflyServer.service.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tofly.toflyServer.utils.UrlUtil;
import com.tofly.toflyServer.utils.Utils;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class AsyncService {
	
	/**
	 * 获取图层列表具体信息
	 * @param arrays
	 * @param url
	 * @param num
	 * @return
	 * @throws InterruptedException
	 */
	@Async("getUrlExecutor")
    public CompletableFuture<List<JSONObject>> getLayerJson(JSONArray arrays, String url) throws InterruptedException {
		
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        CredentialsProvider provider = new BasicCredentialsProvider();
        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
        UsernamePasswordCredentials credentials = new UsernamePasswordCredentials("admin", "geoserver");
        provider.setCredentials(scope, credentials);
        httpClientBuilder.setDefaultCredentialsProvider(provider);
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        
        //返回数据源
        List<JSONObject> resultList =new ArrayList<JSONObject>();
        
        try { 
        	for(int i=0;i<arrays.size();i++) {
	    		JSONObject obj = arrays.getJSONObject(i);
	    		
		        HttpGet httpGet = null;
		        HttpResponse httpResponse = null;
		        HttpEntity entity = null;
		        JSONObject resultJson =null;
		        JSONObject returnJson =null;
		        httpGet = new HttpGet(url+"/"+obj.getString("name"));
    	        
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                resultJson = JSONObject.parseObject(EntityUtils.toString(entity));
	                
	                JSONObject param = resultJson.getJSONObject("layer");
	                JSONObject param1 = param.getJSONObject("resource");
	                for (Map.Entry entry : param1.entrySet()) {
	                	if(entry.getKey().toString().equals("href")) {
	                		String urlJson = UrlUtil.loadJson(entry.getValue().toString());
	                		returnJson = JSONObject.parseObject(urlJson);
	                	}
	                }	      
	                resultList.add(returnJson);
	            }
    		}        
        	closeableHttpClient.close();
        
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
	    return CompletableFuture.completedFuture(resultList);
    }
	
	
	
	@Async("getUrlExecutor")
    public CompletableFuture<List<JSONObject>> getDataSourceJson(JSONArray arrays, String url) throws InterruptedException {
		
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        CredentialsProvider provider = new BasicCredentialsProvider();
        AuthScope scope = new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT, AuthScope.ANY_REALM);
        UsernamePasswordCredentials credentials = new UsernamePasswordCredentials("admin", "geoserver");
        provider.setCredentials(scope, credentials);
        httpClientBuilder.setDefaultCredentialsProvider(provider);
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        
        //返回数据源
        List<JSONObject> resultList =new ArrayList<JSONObject>();
        
        try { 
        	for(int i=0;i<arrays.size();i++) {
	    		JSONObject obj = arrays.getJSONObject(i);
	    		
		        HttpGet httpGet = null;
		        HttpResponse httpResponse = null;
		        HttpEntity entity = null;
		        JSONObject resultJson =null;
		        JSONObject returnJson =null;
		        
		        httpGet = new HttpGet(url+"/"+obj.getString("name")+"/datastores");
    	        
	            httpResponse = closeableHttpClient.execute(httpGet);
	            entity = httpResponse.getEntity();
	            if( entity != null ){
	                resultJson = JSONObject.parseObject(EntityUtils.toString(entity));
	                JSONObject param = resultJson.getJSONObject("dataStores");	  
	                if(null!=param) {
	                	JSONArray param1 = param.getJSONArray("dataStore");
	  	                
	  	                for(int j=0;j<param1.size();j++){
	  	                    JSONObject job = param1.getJSONObject(j); 
	  	                    for (Map.Entry entry : job.entrySet()) {
	  		                	if(entry.getKey().toString().equals("href")) {
	  		                		String urlJson = UrlUtil.loadJson(entry.getValue().toString());
	  		                		returnJson = JSONObject.parseObject(urlJson);
	  		                	}
	  		                }	      
	  		                resultList.add(returnJson);
	  	                }
	                }
	            }
    		}        
        	closeableHttpClient.close();
        
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
	    return CompletableFuture.completedFuture(resultList);
    }
	
	
	

	
}